@@ -1,7 +1,7 @@
 #!/bin/bash
-set -eup pipeline
+set -euo pipefail
 
-mkdir -p intermediate/ summary/
+mkdir -p intermediate_defaults/ intermediate_long/ summary/ tmp_merged/
 
 if [ -f taxdmp_2019-12-01/names.dmp ]; then
     echo "Already have taxdmp_2019-12-01/"
@@ -24,7 +24,7 @@ for ACC in `grep ^SRR PRJNA417859.tsv| cut -f 1`; do
     for URL in `grep ^$ACC PRJNA417859.tsv | cut -f 6 | sed "s/;/ /g"`; do
         NAME=${URL##*/}
         FILE=raw_data/$NAME
-        # Avoiding leaving partial FASTQ if wget is interupted
+        # Avoiding leaving partial FASTQ if wget is interrupted
         rm -rf $FILE.tmp
         if [ -f $FILE ]; then
             echo "Already have $FILE"
