@@ -11,6 +11,7 @@ set -eu
 # Note not using "set -o pipefail" until after check error message with grep
 
 export TMP=${TMP:-/tmp/thapbi_pict}/classify
+rm -rf $TMP
 mkdir -p $TMP
 
 echo "================="
@@ -25,7 +26,8 @@ set -o pipefail
 export DB=$TMP/curated.sqlite
 if [ ! -f $DB ]; then
     thapbi_pict load-tax -d $DB -t new_taxdump_2019-09-01
-    thapbi_pict import -d $DB -i database/Phytophthora_ITS1_curated.fasta -s ";"
+    thapbi_pict import -d $DB -i database/Phytophthora_ITS1_curated.fasta -s ";" \
+                -k ITS1 -l GAAGGTGAAGTCGTAACAAGG -r GCARRGACTTTCGTCCCYRC
 fi
 
 # Passing one filename; default output dir:
@@ -37,20 +39,13 @@ if [ "`grep -c -v '^#' $TMP/input/Phytophthora_ITS1_curated.identity.tsv`" -ne "
 
 rm -rf $TMP/DNAMIX_S95_L001.identity.tsv
 rm -rf $TMP/thapbi_onebp
-rm -rf $TMP/thapbi_swarm
-rm -rf $TMP/thapbi_blast
 rm -rf $TMP/thapbi_1s3g
 mkdir -p $TMP/thapbi_onebp
-mkdir -p $TMP/thapbi_swarm
-mkdir -p $TMP/thapbi_blast
 mkdir -p $TMP/thapbi_1s3g
 
 # Explicitly setting output directory, would be here anyway:
 thapbi_pict classify -m identity -d $DB -i tests/prepare-reads/DNAMIX_S95_L001.fasta -o $TMP/
 thapbi_pict classify -m onebp -d $DB -i tests/prepare-reads/DNAMIX_S95_L001.fasta -o $TMP/thapbi_onebp
-thapbi_pict classify -m blast -d $DB -i tests/prepare-reads/DNAMIX_S95_L001.fasta -o $TMP/thapbi_blast
-thapbi_pict classify -m swarm -d $DB -i tests/prepare-reads/DNAMIX_S95_L001.fasta -o $TMP/thapbi_swarm
-thapbi_pict classify -m swarmid -d $DB -i tests/prepare-reads/DNAMIX_S95_L001.fasta
 thapbi_pict classify -m 1s3g -d $DB -i tests/prepare-reads/DNAMIX_S95_L001.fasta -o $TMP/1s3g
 
 # Passing one directory name (should get all 2 FASTA files):
@@ -63,26 +58,38 @@ if [ "`ls -1 $TMP/duo/*.identity.tsv | wc -l`" -ne "2" ]; then echo "Expected 4
 
 # Test using sequences from a single isolate control,
 rm -rf $TMP/P-infestans-T30-4.*.tsv
-for M in identity onebp substr blast swarm swarmid 1s3g; do
+methods=(identity onebp substr 1s3g)
+if [ -x "$(command -v blastn)" ]; then methods+=(blast); fi
+for M in "${methods[@]}"; do
     echo "Checking single isolate control with $M"
     thapbi_pict classify -d $DB -i tests/classify/P-infestans-T30-4.fasta -o $TMP/ -m $M
     diff $TMP/P-infestans-T30-4.$M.tsv tests/classify/P-infestans-T30-4.$M.tsv
 done
 
 rm -rf $TMP/hmm_trim.*.tsv
-# Swarm classifier can't cope with multiple HMM hits...
-for M in identity onebp blast 1s3g; do
+methods=(identity onebp 1s3g)
+if [ -x "$(command -v blastn)" ]; then methods+=(blast); fi
+for M in "${methods[@]}"; do
     echo "Checking HMM trim corner cases with $M"
     thapbi_pict classify -d $DB -i tests/classify/hmm_trim.fasta -o $TMP/ -m $M -a 50
     diff $TMP/hmm_trim.$M.tsv tests/classify/hmm_trim.$M.tsv
 done
 
+methods=(identity onebp 1s2g 1s3g 1s4g 1s5g)
+if [ -x "$(command -v blastn)" ]; then methods+=(blast); fi
+for M in "${methods[@]}"; do
+    # Using default DB
+    echo "Checking genus corner cases with $M"
+    thapbi_pict classify -i tests/classifier/corner_cases_query.fasta -o $TMP/ -m $M -k ITS1
+    diff $TMP/corner_cases_query.$M.tsv tests/classifier/corner_cases_query.$M.tsv
+done
+
 for EXAMPLE in P_bilorbang P_vulcanica genus_boundary; do
 
     DB=$TMP/${EXAMPLE}.sqlite
     rm -rf $DB $TMP/${EXAMPLE}_query.*
     set -x
-    thapbi_pict import -d $DB -i tests/classifier/${EXAMPLE}.fasta -x
+    thapbi_pict import -d $DB -i tests/classifier/${EXAMPLE}.fasta -x -k ITS1 -l NNN -r NNN
     diff tests/classifier/${EXAMPLE}.fasta <(thapbi_pict dump -m -f fasta -d $DB)
     set +x
     for M in identity onebp 1s2g 1s3g 1s4g 1s5g; do
