@@ -13,6 +13,7 @@ set -o pipefail
 # Note all tests here (initially) using default database:
 
 export TMP=${TMP:-/tmp/thapbi_pict}/conflicts
+rm -rf $TMP
 mkdir -p $TMP
 
 echo "=================="
@@ -28,7 +29,7 @@ diff $TMP/conflicts.tsv tests/conflicts/default.tsv
 # Only 6 FASTA records, but two are double entries so want 8 here
 export DB=$TMP/dup_seqs.sqlite
 rm -rf $DB
-thapbi_pict import -x -d $DB -i tests/curated-import/dup_seqs.fasta -c ncbi -s $'\001'
+thapbi_pict import -x -d $DB -i tests/curated-import/dup_seqs.fasta -c ncbi -s $'\001' -k ITS1 -l "N" -r "N"
 thapbi_pict conflicts -d $TMP/dup_seqs.sqlite -o $TMP/dup_seqs.tsv
 diff $TMP/dup_seqs.tsv tests/conflicts/dup_seqs.tsv
 
