@@ -14,12 +14,12 @@ echo "==================="
 echo "Checking edit-graph"
 echo "==================="
 set -x
-thapbi_pict edit-graph 2>&1 | grep "If not using -i / --input argument, require -s / --showdb"
+thapbi_pict edit-graph 2>&1 | grep "If not using -i / --input argument, require -k / --marker"
 thapbi_pict edit-graph -d '' 2>&1 | grep "Require -d / --database and/or -i / --input argument"
 set -o pipefail
 
 # No database, small FASTA file, have to use explicit total abundance threshold
-if [ `thapbi_pict edit-graph -d '' -i tests/prepare-reads/DNAMIX_S95_L001.fasta -t 200 -m - | grep -c "<edge "` -ne 1 ]; then echo echo "Wrong edge count"; false; fi
+diff --strip-trailing-cr tests/edit-graph/DNAMIX_S95_L001.xgmml <(thapbi_pict edit-graph -d '' -i tests/prepare-reads/DNAMIX_S95_L001.fasta -t 200 -m - )
 # Loaded 7 unique sequences from 1 FASTA files.
 # Minimum total abundance threshold 200 left 7 sequences from FASTA files.
 # Computed 42 Levenshtein edit distances between 7 sequences.
@@ -28,6 +28,7 @@ if [ `thapbi_pict edit-graph -d '' -i tests/prepare-reads/DNAMIX_S95_L001.fasta
 # 1
 
 # Same example as above with default xgmml output, but here different output formats:
+diff --strip-trailing-cr tests/edit-graph/DNAMIX_S95_L001.tsv <(thapbi_pict edit-graph -d '' -i tests/prepare-reads/DNAMIX_S95_L001.fasta -t 200 -m - -f matrix)
 if [ `thapbi_pict edit-graph -d '' -i tests/prepare-reads/DNAMIX_S95_L001.fasta -t 200 -m - -f graphml | grep -c "<edge "` -ne 1 ]; then echo echo "Wrong edge count"; false; fi
 if [ `thapbi_pict edit-graph -d '' -i tests/prepare-reads/DNAMIX_S95_L001.fasta -t 200 -m - -f gexf | grep -c "<edge "` -ne 1 ]; then echo echo "Wrong edge count"; false; fi
 if [ `thapbi_pict edit-graph -d '' -i tests/prepare-reads/DNAMIX_S95_L001.fasta -t 200 -m - -f gml | grep -c "  edge \["` -ne 1 ]; then echo echo "Wrong edge count"; false; fi
@@ -37,7 +38,7 @@ thapbi_pict edit-graph -d '' -i tests/prepare-reads/DNAMIX_S95_L001.fasta -t 200
 
 
 # No database, generic FASTA file, have to use explicit abundance thresholds of 1:
-if [ `thapbi_pict edit-graph -d '' -i database/Phytophthora_ITS1_curated.fasta -a 1 -t 1 -m - | grep -c "<node "` -ne 205 ]; then echo "Wrong node count"; false; fi
+if [ `thapbi_pict edit-graph -d '' -i database/Phytophthora_ITS1_curated.fasta -a 1 -t 1 -m - | grep -c "<node "` -ne 236 ]; then echo "Wrong node count"; false; fi
 # WARNING: Sequence(s) in database/Phytophthora_ITS1_curated.fasta not using MD5_abundance naming
 # Loaded 167 unique sequences from 1 FASTA files.
 # Minimum total abundance threshold 1 left 167 sequences from FASTA files.
