@@ -1,4 +1,4 @@
-# Copyright 2018-2021 by Peter Cock, The James Hutton Institute.
+# Copyright 2018-2022 by Peter Cock, The James Hutton Institute.
 # All rights reserved.
 # This file is part of the THAPBI Phytophthora ITS1 Classifier Tool (PICT),
 # and is released under the "MIT License Agreement". Please see the LICENSE
@@ -10,45 +10,83 @@ This implements the ``thapbi_pict assess ...`` command.
 import sys
 from collections import Counter
 
+from Bio.SeqIO.FastaIO import SimpleFastaParser
+
 from .db_orm import connect_to_db
-from .db_orm import SequenceSource
+from .db_orm import MarkerDef
+from .db_orm import SeqSource
 from .db_orm import Taxonomy
-from .utils import abundance_from_read_name
-from .utils import find_paired_files
+from .prepare import load_fasta_header
+from .utils import file_to_sample_name
+from .utils import find_requested_files
 from .utils import genus_species_name
+from .utils import md5seq
 from .utils import parse_species_tsv
 from .utils import species_level
 from .utils import split_read_name_abundance
 
 
-def sp_in_tsv(classifier_file, min_abundance):
+def load_tsv(mapping, classifier_file, min_abundance):
+    """Update dict mapping of (marker, MD5) to semi-colon separated species string."""
+    for marker, name, _taxid, sp in parse_species_tsv(
+        classifier_file, min_abundance, req_species_level=True, allow_wildcard=False
+    ):
+        if "_" not in name:
+            sys.exit(f"ERROR: Bad entry {name} in {classifier_file}")
+        if not marker:
+            sys.exit(f"ERROR: Missing marker in {classifier_file} header")
+        # wasteful as parse_species_tsv could do it?
+        md5 = split_read_name_abundance(name)[0]
+        if (marker, md5) in mapping:
+            if mapping[marker, md5] != sp:
+                sys.exit(
+                    f"ERROR: Inconsistent classification of {marker} {md5},"
+                    f"{mapping[marker, md5]} vs {sp}"
+                )
+        else:
+            mapping[marker, md5] = sp
+    return mapping
+
+
+def sp_for_sample(fasta_files, min_abundance, pooled_sp):
+    """Return semi-colon separated species string from FASTA files via dict."""
+    answer = set()
+    for fasta_file in fasta_files:
+        try:
+            marker = load_fasta_header(fasta_file)["marker"]
+        except KeyError:
+            sys.exit(f"ERROR: Missing marker in header of {fasta_file}")
+        with open(fasta_file) as handle:
+            for title, seq in SimpleFastaParser(handle):
+                if "_" not in title:
+                    sys.exit(f"ERROR: Bad entry {title} in {fasta_file}")
+                md5, a = split_read_name_abundance(title.split(None, 1)[0])
+                assert md5 == md5seq(seq)
+                if a < min_abundance:
+                    continue
+                sp = pooled_sp[marker, md5]
+                if sp:
+                    answer.update(sp.split(";"))
+    assert "" not in answer
+    return ";".join(sorted(answer))
+
+
+def sp_in_tsv(classifier_files, min_abundance):
     """Return semi-colon separated list of species in column 2.
 
     Will ignore genus level predictions.
     """
     species = set()
-    for line in open(classifier_file):
-        if line.startswith("#") or not line.strip():
-            continue
-        try:
-            name, taxid, sp, etc = line.split("\t", 3)
-        except ValueError:
-            sys.exit(f"ERROR: Wrong field count in {classifier_file}\n")
-        if not sp:
-            continue
-        # Using "*" as name (without abundance) as wildcard
-        if (
-            name != "*"
-            and min_abundance > 1
-            and abundance_from_read_name(name) < min_abundance
+    for classifier_file in classifier_files:
+        for _marker, _name, _taxid, sp in parse_species_tsv(
+            classifier_file, min_abundance, req_species_level=True, allow_wildcard=True
         ):
-            continue
-        species.update(sp.split(";"))
+            species.update(sp.split(";"))
     return ";".join(sorted(_ for _ in species if species_level(_)))
 
 
 def tally_files(expected_file, predicted_file, min_abundance=0):
-    """Make dictionary tally confusion matrix of species assignements.
+    """Make dictionary tally confusion matrix of species assignments.
 
     Rather than the values simply being an integer count, they are
     the set of MD5 identifiers (take the length for the count).
@@ -63,32 +101,32 @@ def tally_files(expected_file, predicted_file, min_abundance=0):
             sorted(parse_species_tsv(expected_file, req_species_level=True)),
             sorted(parse_species_tsv(predicted_file, req_species_level=True)),
         ):
-            if not expt[0] == pred[0]:
+            if not expt[1] == pred[1]:
                 sys.exit(
                     f"ERROR: Sequence name mismatch in {expected_file} vs"
-                    f" {predicted_file}, {expt[0]} vs {pred[0]}\n"
+                    f" {predicted_file}, {expt[1]} vs {pred[1]}\n"
                 )
-            md5, abundance = split_read_name_abundance(expt[0])
+            md5, abundance = split_read_name_abundance(expt[1])
             if min_abundance > 1 and abundance < min_abundance:
                 continue
             # TODO: Look at taxid?
             # Should now have (possibly empty) string of genus-species;...
-            for sp in expt[2].split(";"):
+            for sp in expt[3].split(";"):
                 if sp:
                     assert species_level(sp), (
-                        f"Expectation {expt[2]} is not all species level from"
+                        f"Expectation {expt[3]} is not all species level from"
                         f" {expected_file}"
                     )
-            for sp in pred[2].split(";"):
+            for sp in pred[3].split(";"):
                 if sp:
                     assert species_level(sp), (
-                        f"Prediction {pred[2]} is not all species level from"
+                        f"Prediction {pred[3]} is not all species level from"
                         f" {predicted_file}"
                     )
             try:
-                counter[expt[2], pred[2]].add(md5)
+                counter[expt[3], pred[3]].add(md5)
             except KeyError:
-                counter[expt[2], pred[2]] = {md5}
+                counter[expt[3], pred[3]] = {md5}
     except ValueError as e:
         # This is used for single sample controls,
         # where all reads are expected to be from species X.
@@ -103,13 +141,13 @@ def tally_files(expected_file, predicted_file, min_abundance=0):
         for pred in parse_species_tsv(
             predicted_file, min_abundance, req_species_level=True
         ):
-            md5, abundance = split_read_name_abundance(pred[0])
+            md5, abundance = split_read_name_abundance(pred[1])
             if min_abundance > 1 and abundance < min_abundance:
                 continue
             try:
-                counter[expt_sp_genus, pred[2]].add(md5)
+                counter[expt_sp_genus, pred[3]].add(md5)
             except KeyError:
-                counter[expt_sp_genus, pred[2]] = {md5}
+                counter[expt_sp_genus, pred[3]] = {md5}
     return counter
 
 
@@ -239,12 +277,12 @@ def save_confusion_matrix(tally, db_sp_list, sp_list, filename, exp_total, debug
 
 
 def extract_binary_tally(class_name, tally):
-    """Extact single-class TP, FP, FN, TN from multi-class confusion tally.
+    """Extract single-class TP, FP, FN, TN from multi-class confusion tally.
 
     Reduces the mutli-class expectation/prediction to binary - did they
     include the class of interest, or not?
 
-    Returns a 4-tuple of values, True Positives (TP), False Positves (FP),
+    Returns a 4-tuple of values, True Positives (TP), False Positives (FP),
     False Negatives (FN), True Negatives (TN), which sum to the tally total.
     """
     bt = Counter()
@@ -259,10 +297,10 @@ def extract_global_tally(tally, sp_list):
     If the input data has no negative controls, all there will be no
     true negatives (TN).
 
-    Returns a 4-tuple of values, True Positives (TP), False Positves (FP),
+    Returns a 4-tuple of values, True Positives (TP), False Positives (FP),
     False Negatives (FN), True Negatives (TN).
 
-    These values are analagous to the classical binary classifier approach,
+    These values are analogous to the classical binary classifier approach,
     but are NOT the same. Even if applied to single class expected and
     predicted values, results differ:
 
@@ -328,18 +366,91 @@ def main(
     assess_output,
     map_output,
     confusion_output,
+    marker=None,
     ignore_prefixes=None,
     debug=False,
 ):
-    """Implement the (sample/species level) ``thapbi_pict assess`` command."""
+    """Implement the (sample/species level) ``thapbi_pict assess`` command.
+
+    The inputs argument is a list of filenames and/or folders.
+
+    * To know what species are expected for example sample, we need one known
+      TSV per sample (which can use wildcards), or a multi-sample known TSV
+      which much use full per-sequence entries.
+    * To know which sequences are in which samples, we need one FASTA file per
+      sample per marker.
+    * To know what species were predicted for each sequence, we need TSV files
+      for each marker (this can be a combined file for all reads per marker
+      as produced by the pipeline command, or per sample).
+    """
     assert isinstance(inputs, list), inputs
 
-    input_list = find_paired_files(
-        inputs, f".{method}.tsv", f".{known}.tsv", ignore_prefixes, debug, strict=False
-    )
+    # TODO - Rationalise this, too many ways to give the inputs!
 
-    if not input_list:
-        sys.exit(f"ERROR: Found no pairs *.{method}.tsv with *.{known}.tsv")
+    fasta_files = {}  # map sample name to list of FASTA filenames
+    for fasta_file in find_requested_files(
+        inputs, ".fasta", ignore_prefixes, debug=debug
+    ):
+        sample = file_to_sample_name(fasta_file)
+        if sample.endswith(".all_reads"):
+            sys.exit(f"ERROR: Need per sample FASTA files, not {fasta_file}")
+        try:
+            fasta_files[sample].append(fasta_file)
+        except KeyError:
+            fasta_files[sample] = [fasta_file]
+
+    known_tsv = {}
+    pooled_known = None
+    for tsv_file in find_requested_files(
+        inputs, f".{known}.tsv", ignore_prefixes, debug
+    ):
+        sample = file_to_sample_name(tsv_file)
+        if sample.endswith(".all_reads"):
+            if None in known:
+                sys.exit(f"ERROR: More than one pooled TSV for {method}")
+            load_tsv(pooled_known, tsv_file, min_abundance)
+        elif sample in fasta_files or not fasta_files:
+            try:
+                known_tsv[sample].append(tsv_file)
+            except KeyError:
+                known_tsv[sample] = [tsv_file]
+        else:
+            sys.stderr.write(f"WARNING: Ignoring {tsv_file}\n")
+    if len(fasta_files) > 1 and len(known_tsv) == 1 and None not in known_tsv:
+        sys.exit("ERROR: Missing some {known} TSV files")
+
+    if pooled_known and known_tsv:
+        sys.exit(f"ERROR: Use either pooled {known} TSV, or per sample TSV")
+
+    if known_tsv and fasta_files:
+        samples = sorted(set(fasta_files).intersection(known_tsv))
+    elif known_tsv:
+        samples = sorted(known_tsv)
+    else:
+        samples = sorted(fasta_files)
+    if not samples:
+        sys.exit("ERROR: Could not determine a sample list")
+    if debug:
+        sys.stderr.write(f"Assessing {len(samples)} samples\n")
+
+    method_tsv = {}
+    pooled_method = {}
+    for tsv_file in find_requested_files(
+        inputs, f".{method}.tsv", ignore_prefixes, debug
+    ):
+        sample = file_to_sample_name(tsv_file)
+        if sample.endswith(".all_reads"):
+            load_tsv(pooled_method, tsv_file, min_abundance)
+        elif sample in samples:
+            try:
+                method_tsv[sample].append(tsv_file)
+            except KeyError:
+                method_tsv[sample] = [tsv_file]
+        else:
+            sys.stderr.write(f"WARNING: Ignoring {tsv_file}\n")
+
+    if pooled_method and method_tsv:
+        sys.exit(f"ERROR: Use either pooled {known} TSV, or per sample TSV")
 
     # Connect to the DB,
     Session = connect_to_db(db_url, echo=False)  # echo=debug is too distracting now
@@ -347,8 +458,13 @@ def main(
     view = (
         session.query(Taxonomy)
         .distinct(Taxonomy.genus, Taxonomy.species)
-        .join(SequenceSource)
+        .join(SeqSource)
     )
+    if marker:
+        # TODO - Check this is actually in the DB?
+        view = view.join(MarkerDef, SeqSource.marker_definition).filter(
+            MarkerDef.name == marker
+        )
     db_sp_list = sorted(
         {genus_species_name(t.genus, t.species) for t in view if t.species}
     )
@@ -359,14 +475,20 @@ def main(
     file_count = 0
     global_tally = {}
 
-    for predicted_file, expected_file in input_list:
-        if debug:
-            sys.stderr.write(f"Assessing {predicted_file} vs {expected_file}\n")
+    for sample in samples:
         file_count += 1
-        expt = sp_in_tsv(expected_file, min_abundance)
-        pred = sp_in_tsv(predicted_file, min_abundance)
+        if sample in known_tsv:
+            expt = sp_in_tsv(known_tsv[sample], min_abundance)
+        elif pooled_known:
+            expt = sp_for_sample(fasta_files[sample], min_abundance, pooled_known)
+        else:
+            sys.exit(f"ERROR: Missing {known} results for {sample}")
+        if sample in fasta_files:
+            pred = sp_for_sample(fasta_files[sample], min_abundance, pooled_method)
+        else:
+            pred = sp_in_tsv(method_tsv[sample], min_abundance)
         global_tally[expt, pred] = global_tally.get((expt, pred), 0) + 1
-    assert len(input_list) == sum(global_tally.values())
+    assert len(samples) == sum(global_tally.values())
 
     if debug:
         sys.stderr.write(f"DEBUG: Assessing {sum(global_tally.values())} predictions\n")
@@ -394,7 +516,7 @@ def main(
         f" ({len(sp_list)} species)\n"
     )
 
-    assert file_count == len(input_list)
+    assert file_count == len(samples)
 
     if not file_count:
         sys.exit("ERROR: Could not find files to assess\n")
