@@ -1,4 +1,4 @@
-# Copyright 2018-2021 by Peter Cock, The James Hutton Institute.
+# Copyright 2018-2022 by Peter Cock, The James Hutton Institute.
 # All rights reserved.
 # This file is part of the THAPBI Phytophthora ITS1 Classifier Tool (PICT),
 # and is released under the "MIT License Agreement". Please see the LICENSE
@@ -11,11 +11,36 @@ import subprocess
 import sys
 import time
 from collections import Counter
+from keyword import iskeyword
 
 from Bio.Data.IUPACData import ambiguous_dna_values
 from Bio.SeqIO.FastaIO import SimpleFastaParser
 
 
+def valid_marker_name(text):
+    """Check the proposed string valid for use as a marker name.
+
+    Want to be able to use the string for file or directory names, and also
+    column names etc in reports. At very least should reject whitespace, line
+    breaks, and slashes.
+
+    Also rejecting all digits, as might want to accept integers as argument
+    (e.g. cluster array job mapping task numbers to marker numbers).
+
+    Also rejecting the underscore, as may want to use it as a field separator
+    in sequence names (e.g. ``marker_md5_abundance``), and full stop as may
+    use it as a field separator in filenames.
+
+    May want to relax this later, thus defining this central function.
+    """
+    return (
+        text.replace("-", "").isalnum()
+        and not iskeyword(text)
+        and not text.isdigit()
+        and text != "pooled"
+    )
+
+
 def primer_clean(primer):
     """Handle non-IUPAC entries in primers, maps I for inosine to N.
 
@@ -30,6 +55,15 @@ def primer_clean(primer):
     return primer.upper().replace("I", "N")
 
 
+def reject_species_name(species):
+    """Reject species names like 'environmental samples' or 'uncultured ...'."""
+    return (
+        not species
+        or species.split(None, 1)[0] in ("unclassified", "uncultured", "unidentified")
+        or species == "environmental samples"
+    )
+
+
 def genus_species_name(genus, species):
     """Return name, genus with species if present.
 
@@ -74,7 +108,7 @@ def onebp_substitutions(seq):
     variants = set()
     for i in range(len(seq)):
         for s in "ACGT":
-            # One base substitions
+            # One base substitutions
             variants.add(seq[:i] + s + seq[i + 1 :])
     variants.remove(seq)
     return variants
@@ -114,6 +148,8 @@ def onebp_variants(seq):
     """Generate all 1bp variants of the sequence (substitution, deletion or insertion).
 
     Assumes unambiguous IUPAC codes A, C, G, T only.
+
+    Will not return the input sequence!
     """
     seq = seq.upper()
     variants = set()
@@ -121,7 +157,7 @@ def onebp_variants(seq):
         # One base deletion
         variants.add(seq[:i] + seq[i + 1 :])
         for s in "ACGT":
-            # One base substitions
+            # One base substitutions
             variants.add(seq[:i] + s + seq[i + 1 :])
             # One base insertions
             variants.add(seq[:i] + s + seq[i:])
@@ -294,8 +330,8 @@ def cmd_as_string(cmd):
     Intended for using in debugging or error messages.
     """
     if isinstance(cmd, list):
-        # Quote any entries with spaces
-        return " ".join('"%s"' % _ if " " in _ else _ for _ in cmd)
+        # Quote any entries with spaces or semi-colons
+        return " ".join('"%s"' % _ if (" " in _ or ";" in _) else _ for _ in cmd)
     else:
         return cmd
 
@@ -320,18 +356,16 @@ def run(cmd, debug=False, attempts=1):
             if isinstance(cmd, list):
                 return subprocess.run(
                     cmd,
-                    stdout=subprocess.PIPE,
-                    stderr=subprocess.PIPE,
-                    universal_newlines=True,
+                    capture_output=True,
+                    text=True,
                     check=True,
                 )
             else:
                 return subprocess.run(
                     cmd,
                     shell=True,
-                    stdout=subprocess.PIPE,
-                    stderr=subprocess.PIPE,
-                    universal_newlines=True,
+                    capture_output=True,
+                    text=True,
                     check=True,
                 )
         except subprocess.CalledProcessError as e:
@@ -447,7 +481,7 @@ def find_requested_files(
     for x in filenames_or_folders:
         if os.path.isdir(x):
             if debug:
-                sys.stderr.write(f"Walking directory {x!r}\n")
+                sys.stderr.write(f"DEBUG: Walking directory {x}\n")
             for root, _, files in os.walk(x, followlinks=True):
                 for f in files:
                     if f.endswith(ext):
@@ -463,14 +497,11 @@ def find_requested_files(
             if x.endswith(ext):
                 if ignore_prefixes and x.startswith(ignore_prefixes):
                     if debug:
-                        sys.stderr.write("DEBUG: Ignoring %s\n" % os.path.join(root, x))
+                        sys.stderr.write(f"DEBUG: Due to prefix ignoring {x}\n")
                     continue
                 answer.append(x)
             elif debug:
-                sys.stderr.write(
-                    f"DEBUG: Ignoring {x!r} as does not have"
-                    f" expected extension {ext!r}."
-                )
+                sys.stderr.write(f"DEBUG: Due to extension ignoring {x}\n")
         else:
             sys.exit(f"ERROR: {x!r} is not a file or a directory\n")
     # Warn if there were duplicates?
@@ -539,11 +570,21 @@ def find_paired_files(
     return input_list
 
 
-def parse_species_tsv(tabular_file, min_abundance=0, req_species_level=False):
-    """Parse file of species assignments/predictions by sequence."""
+def parse_species_tsv(
+    tabular_file, min_abundance=0, req_species_level=False, allow_wildcard=False
+):
+    """Parse file of species assignments/predictions by sequence.
+
+    Yields tuples of marker name (from the file header line), sequence name,
+    taxid, and genus_species.
+    """
+    marker = None
     with open(tabular_file) as handle:
         for line in handle:
             if line.startswith("#"):
+                parts = line[1:].split("\t")
+                if parts[0].endswith("/sequence-name") and parts[1] == "taxid":
+                    marker = parts[0][:-14]
                 continue
             if line.count("\t") == 2:
                 name, taxid, genus_species = line.rstrip("\n").split("\t", 3)
@@ -554,20 +595,30 @@ def parse_species_tsv(tabular_file, min_abundance=0, req_species_level=False):
                     f"ERROR: {tabular_file} is not 3 or 4 column TSV"
                     f" (name, taxid, genus-species, optional note):\n{line}"
                 )
-            if name == "*":
+            if name == "*" and not allow_wildcard:
                 raise ValueError("Wildcard species name found")
-            if min_abundance > 1 and abundance_from_read_name(name) < min_abundance:
+            if (
+                min_abundance > 1
+                and abundance_from_read_name(name) < min_abundance
+                and name != "*"
+            ):
                 continue
             if req_species_level:
-                assert taxid.count(";") == genus_species.count(";"), line
-                wanted = [
-                    (t, s)
-                    for (t, s) in zip(taxid.split(";"), genus_species.split(";"))
-                    if species_level(s)
-                ]
-                taxid = ";".join(t for (t, s) in wanted)
-                genus_species = ";".join(s for (t, s) in wanted)
-            yield name, taxid, genus_species
+                if taxid and taxid != "0":
+                    assert taxid.count(";") == genus_species.count(";"), line
+                    wanted = [
+                        (t, s)
+                        for (t, s) in zip(taxid.split(";"), genus_species.split(";"))
+                        if species_level(s)
+                    ]
+                    taxid = ";".join(t for (t, s) in wanted)
+                    genus_species = ";".join(s for (t, s) in wanted)
+                else:
+                    taxid = ""
+                    genus_species = ";".join(
+                        s for s in genus_species.split(";") if species_level(s)
+                    )
+            yield marker, name, taxid, genus_species
 
 
 def load_metadata(
@@ -664,6 +715,19 @@ def load_metadata(
     else:
         group_col = 0  # Default is the first requested column
 
+    def dequote_line(fields_in_line):
+        return [
+            _[1:-1] if _.count('"') == 2 and _[0] == '"' and _[-1] == '"' else _
+            for _ in fields_in_line
+        ]
+
+    assert dequote_line(["A", "A is B", '"A is also C"', "Not D"]) == [
+        "A",
+        "A is B",
+        "A is also C",
+        "Not D",
+    ]
+
     names = [""] * len(value_cols)  # default
     meta = {}
     if not metadata_encoding:
@@ -685,7 +749,7 @@ def load_metadata(
         line = lines[metadata_name_row - 1]
         if line.startswith("#"):
             line = line[1:]
-        parts = line.rstrip("\n").split("\t")
+        parts = dequote_line(line.rstrip("\n").split("\t"))
         if len(parts) < max(value_cols) + 1:
             sys.exit("ERROR: Not enough columns in metadata name row")
         names = [parts[_].strip() for _ in value_cols]
@@ -698,11 +762,21 @@ def load_metadata(
                 f"DEBUG: Grouping on {names[group_col]} for colour bands\n"
             )
 
-    # Remove header lines,
-    lines = [_ for _ in lines[metadata_name_row:] if not _.startswith("#")]
+    # Remove header lines, and empty lines
+    lines = [
+        _ for _ in lines[metadata_name_row:] if _.strip() and not _.startswith("#")
+    ]
 
     # Break up line into fields
-    lines = [_.rstrip("\n").split("\t") for _ in lines]
+    lines = [dequote_line(_.rstrip("\n").split("\t")) for _ in lines]
+
+    for _ in lines:
+        if len(_) <= sample_col:
+            sys.exit(f"ERROR: Missing sample column {sample_col+1} in metadata")
+        if len(_) <= max(value_cols):
+            sys.exit(
+                f"ERROR: Missing column {max(value_cols)+1} for sample {_[sample_col]}"
+            )
 
     # Select columns of interest
     meta_plus_idx = [[_[i].strip() for i in value_cols + [sample_col]] for _ in lines]
